@@ -23,6 +23,7 @@ import (
 	"errors"
 	"io"
 	"sync"
+	"time"
 
 	"go.uber.org/multierr"
 
@@ -44,8 +45,10 @@ type (
 
 // Message is send on the bus to all subscribed listeners.
 type Message struct {
-	payload payload
-	id      MessageID
+	payload   payload
+	node      string
+	id        MessageID
+	batchMode bool
 }
 
 // ID outputs the MessageID of the Message.
@@ -58,9 +61,29 @@ func (m Message) Data() interface{} {
 	return m.payload
 }
 
+// Node returns the node name of the Message.
+func (m Message) Node() string {
+	return m.node
+}
+
+// BatchModeEnabled returns whether the Message is sent in batch mode.
+func (m Message) BatchModeEnabled() bool {
+	return m.batchMode
+}
+
 // NewMessage returns a new Message with a MessageID and embed data.
 func NewMessage(id MessageID, data interface{}) Message {
-	return Message{id: id, payload: data}
+	return Message{id: id, node: "local", payload: data}
+}
+
+// NewBatchMessageWithNode returns a new Message with a MessageID and NodeID and embed data.
+func NewBatchMessageWithNode(id MessageID, node string, data interface{}) Message {
+	return Message{id: id, node: node, payload: data, batchMode: true}
+}
+
+// NewMessageWithNode returns a new Message with a MessageID and NodeID and embed data.
+func NewMessageWithNode(id MessageID, node string, data interface{}) Message {
+	return Message{id: id, node: node, payload: data}
 }
 
 // MessageListener is the signature of functions that can handle an EventMessage.
@@ -78,6 +101,11 @@ type Publisher interface {
 	Publish(topic Topic, message ...Message) (Future, error)
 }
 
+// Broadcaster allow sending Messages to a Topic and receiving the responses.
+type Broadcaster interface {
+	Broadcast(timeout time.Duration, topic Topic, message Message) ([]Future, error)
+}
+
 type channel chan event
 
 type chType int
@@ -103,6 +131,11 @@ func BiTopic(id string) Topic {
 	return Topic{id: id, typ: chTypeBidirectional}
 }
 
+// String returns the string representation of the Topic.
+func (t Topic) String() string {
+	return t.id
+}
+
 // The Bus allows publish-subscribe-style communication between components.
 type Bus struct {
 	topics map[Topic][]channel
