@@ -32,25 +32,31 @@ import (
 
 type streamRegistryServer struct {
 	databasev1.UnimplementedStreamRegistryServiceServer
-	schemaRegistry metadata.Service
+	schemaRegistry metadata.Repo
 }
 
 func (rs *streamRegistryServer) Create(ctx context.Context,
 	req *databasev1.StreamRegistryServiceCreateRequest,
 ) (*databasev1.StreamRegistryServiceCreateResponse, error) {
-	if err := rs.schemaRegistry.StreamRegistry().CreateStream(ctx, req.GetStream()); err != nil {
+	modRevision, err := rs.schemaRegistry.StreamRegistry().CreateStream(ctx, req.GetStream())
+	if err != nil {
 		return nil, err
 	}
-	return &databasev1.StreamRegistryServiceCreateResponse{}, nil
+	return &databasev1.StreamRegistryServiceCreateResponse{
+		ModRevision: modRevision,
+	}, nil
 }
 
 func (rs *streamRegistryServer) Update(ctx context.Context,
 	req *databasev1.StreamRegistryServiceUpdateRequest,
 ) (*databasev1.StreamRegistryServiceUpdateResponse, error) {
-	if err := rs.schemaRegistry.StreamRegistry().UpdateStream(ctx, req.GetStream()); err != nil {
+	modRevision, err := rs.schemaRegistry.StreamRegistry().UpdateStream(ctx, req.GetStream())
+	if err != nil {
 		return nil, err
 	}
-	return &databasev1.StreamRegistryServiceUpdateResponse{}, nil
+	return &databasev1.StreamRegistryServiceUpdateResponse{
+		ModRevision: modRevision,
+	}, nil
 }
 
 func (rs *streamRegistryServer) Delete(ctx context.Context,
@@ -123,7 +129,7 @@ func groupExist(ctx context.Context, errResource error, metadata *commonv1.Metad
 
 type indexRuleBindingRegistryServer struct {
 	databasev1.UnimplementedIndexRuleBindingRegistryServiceServer
-	schemaRegistry metadata.Service
+	schemaRegistry metadata.Repo
 }
 
 func (rs *indexRuleBindingRegistryServer) Create(ctx context.Context,
@@ -205,7 +211,7 @@ func (rs *indexRuleBindingRegistryServer) Exist(ctx context.Context, req *databa
 
 type indexRuleRegistryServer struct {
 	databasev1.UnimplementedIndexRuleRegistryServiceServer
-	schemaRegistry metadata.Service
+	schemaRegistry metadata.Repo
 }
 
 func (rs *indexRuleRegistryServer) Create(ctx context.Context, req *databasev1.IndexRuleRegistryServiceCreateRequest) (
@@ -281,25 +287,31 @@ func (rs *indexRuleRegistryServer) Exist(ctx context.Context, req *databasev1.In
 
 type measureRegistryServer struct {
 	databasev1.UnimplementedMeasureRegistryServiceServer
-	schemaRegistry metadata.Service
+	schemaRegistry metadata.Repo
 }
 
 func (rs *measureRegistryServer) Create(ctx context.Context, req *databasev1.MeasureRegistryServiceCreateRequest) (
 	*databasev1.MeasureRegistryServiceCreateResponse, error,
 ) {
-	if err := rs.schemaRegistry.MeasureRegistry().CreateMeasure(ctx, req.GetMeasure()); err != nil {
+	modRevision, err := rs.schemaRegistry.MeasureRegistry().CreateMeasure(ctx, req.GetMeasure())
+	if err != nil {
 		return nil, err
 	}
-	return &databasev1.MeasureRegistryServiceCreateResponse{}, nil
+	return &databasev1.MeasureRegistryServiceCreateResponse{
+		ModRevision: modRevision,
+	}, nil
 }
 
 func (rs *measureRegistryServer) Update(ctx context.Context, req *databasev1.MeasureRegistryServiceUpdateRequest) (
 	*databasev1.MeasureRegistryServiceUpdateResponse, error,
 ) {
-	if err := rs.schemaRegistry.MeasureRegistry().UpdateMeasure(ctx, req.GetMeasure()); err != nil {
+	modRevision, err := rs.schemaRegistry.MeasureRegistry().UpdateMeasure(ctx, req.GetMeasure())
+	if err != nil {
 		return nil, err
 	}
-	return &databasev1.MeasureRegistryServiceUpdateResponse{}, nil
+	return &databasev1.MeasureRegistryServiceUpdateResponse{
+		ModRevision: modRevision,
+	}, nil
 }
 
 func (rs *measureRegistryServer) Delete(ctx context.Context, req *databasev1.MeasureRegistryServiceDeleteRequest) (
@@ -355,7 +367,7 @@ func (rs *measureRegistryServer) Exist(ctx context.Context, req *databasev1.Meas
 
 type groupRegistryServer struct {
 	databasev1.UnimplementedGroupRegistryServiceServer
-	schemaRegistry metadata.Service
+	schemaRegistry metadata.Repo
 }
 
 func (rs *groupRegistryServer) Create(ctx context.Context, req *databasev1.GroupRegistryServiceCreateRequest) (
@@ -429,7 +441,7 @@ func (rs *groupRegistryServer) Exist(ctx context.Context, req *databasev1.GroupR
 
 type topNAggregationRegistryServer struct {
 	databasev1.UnimplementedTopNAggregationRegistryServiceServer
-	schemaRegistry metadata.Service
+	schemaRegistry metadata.Repo
 }
 
 func (ts *topNAggregationRegistryServer) Create(ctx context.Context,
