@@ -21,45 +21,50 @@
 package measure
 
 import (
-	"context"
 	"time"
 
 	commonv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/common/v1"
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
-	"github.com/apache/skywalking-banyandb/banyand/metadata"
 	"github.com/apache/skywalking-banyandb/banyand/queue"
-	"github.com/apache/skywalking-banyandb/banyand/tsdb"
-	"github.com/apache/skywalking-banyandb/banyand/tsdb/index"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	"github.com/apache/skywalking-banyandb/pkg/partition"
-	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
 	"github.com/apache/skywalking-banyandb/pkg/query/logical"
+	"github.com/apache/skywalking-banyandb/pkg/schema"
 	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 )
 
 const (
-	plainChunkSize = 1 << 20
-	intChunkNum    = 120
-	intChunkSize   = 4 * 1024
+	maxValuesBlockSize              = 8 * 1024 * 1024
+	maxTimestampsBlockSize          = 8 * 1024 * 1024
+	maxTagFamiliesMetadataSize      = 8 * 1024 * 1024
+	maxUncompressedBlockSize        = 2 * 1024 * 1024
+	maxUncompressedPrimaryBlockSize = 128 * 1024
+
+	maxBlockLength = 8 * 1024
+
+	defaultFlushTimeout = 5 * time.Second
 )
 
-type measure struct {
-	databaseSupplier       tsdb.Supplier
-	l                      *logger.Logger
-	schema                 *databasev1.Measure
-	indexWriter            *index.Writer
-	processorManager       *topNProcessorManager
-	name                   string
-	group                  string
-	entityLocator          partition.EntityLocator
-	indexRules             []*databasev1.IndexRule
-	topNAggregations       []*databasev1.TopNAggregation
-	maxObservedModRevision int64
-	interval               time.Duration
-	shardNum               uint32
+type option struct {
+	mergePolicy  *mergePolicy
+	flushTimeout time.Duration
 }
 
-func (s *measure) startSteamingManager(pipeline queue.Queue, repo metadata.Repo) error {
+type measure struct {
+	databaseSupplier  schema.Supplier
+	l                 *logger.Logger
+	schema            *databasev1.Measure
+	processorManager  *topNProcessorManager
+	name              string
+	group             string
+	indexRules        []*databasev1.IndexRule
+	indexRuleLocators partition.IndexRuleLocator
+	topNAggregations  []*databasev1.TopNAggregation
+	interval          time.Duration
+	shardNum          uint32
+}
+
+func (s *measure) startSteamingManager(pipeline queue.Queue) error {
 	if len(s.topNAggregations) == 0 {
 		return nil
 	}
@@ -69,7 +74,6 @@ func (s *measure) startSteamingManager(pipeline queue.Queue, repo metadata.Repo)
 	s.processorManager = &topNProcessorManager{
 		l:            s.l,
 		pipeline:     pipeline,
-		repo:         repo,
 		m:            s,
 		s:            tagMapSpec,
 		topNSchemas:  s.topNAggregations,
@@ -83,22 +87,10 @@ func (s *measure) GetSchema() *databasev1.Measure {
 	return s.schema
 }
 
-func (s *measure) GetMetadata() *commonv1.Metadata {
-	return s.schema.Metadata
-}
-
 func (s *measure) GetIndexRules() []*databasev1.IndexRule {
 	return s.indexRules
 }
 
-func (s *measure) MaxObservedModRevision() int64 {
-	return s.maxObservedModRevision
-}
-
-func (s *measure) EntityLocator() partition.EntityLocator {
-	return s.entityLocator
-}
-
 func (s *measure) Close() error {
 	if s.processorManager == nil {
 		return nil
@@ -108,11 +100,11 @@ func (s *measure) Close() error {
 
 func (s *measure) parseSpec() (err error) {
 	s.name, s.group = s.schema.GetMetadata().GetName(), s.schema.GetMetadata().GetGroup()
-	s.entityLocator = partition.NewEntityLocator(s.schema.GetTagFamilies(), s.schema.GetEntity())
-	s.maxObservedModRevision = pbv1.ParseMaxModRevision(s.indexRules)
 	if s.schema.Interval != "" {
 		s.interval, err = timestamp.ParseDuration(s.schema.Interval)
 	}
+	s.indexRuleLocators = partition.ParseIndexRuleLocators(s.schema.GetEntity(), s.schema.GetTagFamilies(), s.indexRules)
+
 	return err
 }
 
@@ -122,7 +114,8 @@ type measureSpec struct {
 	topNAggregations []*databasev1.TopNAggregation
 }
 
-func openMeasure(shardNum uint32, db tsdb.Supplier, spec measureSpec, l *logger.Logger) (*measure, error) {
+func openMeasure(shardNum uint32, db schema.Supplier, spec measureSpec, l *logger.Logger, pipeline queue.Queue,
+) (*measure, error) {
 	m := &measure{
 		shardNum:         shardNum,
 		schema:           spec.schema,
@@ -133,15 +126,14 @@ func openMeasure(shardNum uint32, db tsdb.Supplier, spec measureSpec, l *logger.
 	if err := m.parseSpec(); err != nil {
 		return nil, err
 	}
-	ctx := context.WithValue(context.Background(), logger.ContextKey, l)
+	if db == nil {
+		return m, nil
+	}
 
 	m.databaseSupplier = db
-	m.indexWriter = index.NewWriter(ctx, index.WriterOptions{
-		DB:         db,
-		ShardNum:   shardNum,
-		Families:   spec.schema.TagFamilies,
-		IndexRules: spec.indexRules,
-	})
-
+	if startErr := m.startSteamingManager(pipeline); startErr != nil {
+		l.Err(startErr).Str("measure", spec.schema.GetMetadata().GetName()).
+			Msg("fail to start streaming manager")
+	}
 	return m, nil
 }
