@@ -19,25 +19,27 @@ package stream
 
 import (
 	"context"
+	"math"
 	"path"
-	"time"
 
 	"github.com/pkg/errors"
 
 	"github.com/apache/skywalking-banyandb/api/data"
 	commonv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/common/v1"
-	"github.com/apache/skywalking-banyandb/banyand/discovery"
+	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
 	"github.com/apache/skywalking-banyandb/banyand/metadata"
-	"github.com/apache/skywalking-banyandb/banyand/metadata/schema"
+	"github.com/apache/skywalking-banyandb/banyand/observability"
 	"github.com/apache/skywalking-banyandb/banyand/queue"
-	"github.com/apache/skywalking-banyandb/banyand/tsdb"
+	"github.com/apache/skywalking-banyandb/pkg/bus"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	"github.com/apache/skywalking-banyandb/pkg/run"
+	resourceSchema "github.com/apache/skywalking-banyandb/pkg/schema"
 )
 
 var (
-	errEmptyRootPath  = errors.New("root path is empty")
-	errStreamNotExist = errors.New("stream doesn't exist")
+	errEmptyRootPath = errors.New("root path is empty")
+	// ErrStreamNotExist denotes a stream doesn't exist in the metadata repo.
+	ErrStreamNotExist = errors.New("stream doesn't exist")
 )
 
 // Service allows inspecting the stream elements.
@@ -52,31 +54,34 @@ var _ Service = (*service)(nil)
 
 type service struct {
 	schemaRepo    schemaRepo
+	writeListener bus.MessageListener
 	metadata      metadata.Repo
-	pipeline      queue.Queue
-	repo          discovery.ServiceRepo
-	writeListener *writeCallback
+	pipeline      queue.Server
+	localPipeline queue.Queue
 	l             *logger.Logger
-	stopCh        chan struct{}
 	root          string
-	dbOpts        tsdb.DatabaseOpts
+	option        option
 }
 
 func (s *service) Stream(metadata *commonv1.Metadata) (Stream, error) {
 	sm, ok := s.schemaRepo.loadStream(metadata)
 	if !ok {
-		return nil, errors.WithStack(errStreamNotExist)
+		return nil, errors.WithStack(ErrStreamNotExist)
 	}
 	return sm, nil
 }
 
+func (s *service) LoadGroup(name string) (resourceSchema.Group, bool) {
+	return s.schemaRepo.LoadGroup(name)
+}
+
 func (s *service) FlagSet() *run.FlagSet {
 	flagS := run.NewFlagSet("storage")
 	flagS.StringVar(&s.root, "stream-root-path", "/tmp", "the root path of database")
-	flagS.Int64Var(&s.dbOpts.BlockMemSize, "stream-block-mem-size", 8<<20, "block memory size")
-	flagS.Int64Var(&s.dbOpts.SeriesMemSize, "stream-seriesmeta-mem-size", 1<<20, "series metadata memory size")
-	flagS.Int64Var(&s.dbOpts.GlobalIndexMemSize, "stream-global-index-mem-size", 2<<20, "global index memory size")
-	flagS.Int64Var(&s.dbOpts.BlockInvertedIndex.BatchWaitSec, "stream-idx-batch-wait-sec", 1, "index batch wait in second")
+	flagS.DurationVar(&s.option.flushTimeout, "stream-flush-timeout", defaultFlushTimeout, "the memory data timeout of stream")
+	flagS.DurationVar(&s.option.elementIndexFlushTimeout, "element-index-flush-timeout", defaultFlushTimeout, "the elementIndex timeout of stream")
+	s.option.mergePolicy = newDefaultMergePolicy()
+	flagS.Uint64Var(&s.option.mergePolicy.maxFanOutSize, "max-fan-out-size", math.MaxUint64, "the upper bound of a single file size after merge")
 	return flagS
 }
 
@@ -91,71 +96,39 @@ func (s *service) Name() string {
 	return "stream"
 }
 
-func (s *service) PreRun() error {
+func (s *service) Role() databasev1.Role {
+	return databasev1.Role_ROLE_DATA
+}
+
+func (s *service) PreRun(_ context.Context) error {
 	s.l = logger.GetLogger(s.Name())
-	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
-	groups, err := s.metadata.GroupRegistry().ListGroup(ctx)
-	cancel()
+	path := path.Join(s.root, s.Name())
+	observability.UpdatePath(path)
+	s.localPipeline = queue.Local()
+	s.schemaRepo = newSchemaRepo(path, s)
+	// run a serial watcher
+
+	s.writeListener = setUpWriteCallback(s.l, &s.schemaRepo)
+	err := s.pipeline.Subscribe(data.TopicStreamWrite, s.writeListener)
 	if err != nil {
 		return err
 	}
-	s.schemaRepo = newSchemaRepo(path.Join(s.root, s.Name()), s.metadata, s.repo, s.dbOpts, s.l)
-	for _, g := range groups {
-		if g.Catalog != commonv1.Catalog_CATALOG_STREAM {
-			continue
-		}
-		gp, err := s.schemaRepo.StoreGroup(g.Metadata)
-		if err != nil {
-			return err
-		}
-		ctx, cancel = context.WithTimeout(context.Background(), 5*time.Second)
-		schemas, err := s.metadata.StreamRegistry().ListStream(ctx, schema.ListOpt{Group: gp.GetSchema().GetMetadata().Name})
-		cancel()
-		if err != nil {
-			return err
-		}
-		for _, sa := range schemas {
-			if _, innerErr := gp.StoreResource(sa); innerErr != nil {
-				return innerErr
-			}
-		}
-	}
-
-	s.writeListener = setUpWriteCallback(s.l, &s.schemaRepo)
-
-	errWrite := s.pipeline.Subscribe(data.TopicStreamWrite, s.writeListener)
-	if errWrite != nil {
-		return errWrite
-	}
-	return nil
+	return s.localPipeline.Subscribe(data.TopicStreamWrite, s.writeListener)
 }
 
 func (s *service) Serve() run.StopNotify {
-	_ = s.schemaRepo.NotifyAll()
-	// run a serial watcher
-	s.schemaRepo.Watcher()
-
-	s.metadata.StreamRegistry().RegisterHandler(schema.KindGroup|schema.KindStream|schema.KindIndexRuleBinding|schema.KindIndexRule,
-		&s.schemaRepo)
-	return s.stopCh
+	return s.schemaRepo.StopCh()
 }
 
 func (s *service) GracefulStop() {
+	s.localPipeline.GracefulStop()
 	s.schemaRepo.Close()
-	if s.stopCh != nil {
-		close(s.stopCh)
-	}
 }
 
 // NewService returns a new service.
-func NewService(_ context.Context, metadata metadata.Repo, repo discovery.ServiceRepo, pipeline queue.Queue) (Service, error) {
+func NewService(_ context.Context, metadata metadata.Repo, pipeline queue.Server) (Service, error) {
 	return &service{
 		metadata: metadata,
-		repo:     repo,
 		pipeline: pipeline,
-		dbOpts: tsdb.DatabaseOpts{
-			EnableGlobalIndex: true,
-		},
-		stopCh: make(chan struct{}),
 	}, nil
 }
