@@ -47,8 +47,12 @@ var _ = g.DescribeTable("Scanning Measures", verify,
 	g.Entry("filter by an unknown tag", helpers.Args{Input: "tag_filter_unknown", Duration: 25 * time.Minute, Offset: -20 * time.Minute, WantEmpty: true}),
 	g.Entry("group and max", helpers.Args{Input: "group_max", Duration: 25 * time.Minute, Offset: -20 * time.Minute}),
 	g.Entry("group without field", helpers.Args{Input: "group_no_field", Duration: 25 * time.Minute, Offset: -20 * time.Minute}),
-	g.Entry("top 2", helpers.Args{Input: "top", Duration: 25 * time.Minute, Offset: -20 * time.Minute}),
-	g.Entry("bottom 2", helpers.Args{Input: "bottom", Duration: 25 * time.Minute, Offset: -20 * time.Minute}),
+	g.Entry("top 2 by id", helpers.Args{Input: "top", Duration: 25 * time.Minute, Offset: -20 * time.Minute}),
+	g.Entry("bottom 2 by id", helpers.Args{Input: "bottom", Duration: 25 * time.Minute, Offset: -20 * time.Minute}),
+	g.Entry("top 5 by entity id", helpers.Args{Input: "top_entity", Duration: 30 * time.Minute, Offset: -20 * time.Minute}),
+	g.Entry("bottom 5 by entity id", helpers.Args{Input: "bottom_entity", Duration: 30 * time.Minute, Offset: -20 * time.Minute}),
+	g.Entry("top 2 by entity id in svc_1", helpers.Args{Input: "top_entity_svc", Duration: 30 * time.Minute, Offset: -20 * time.Minute}),
+	g.Entry("bottom 2 by entity id in svc_1", helpers.Args{Input: "bottom_entity_svc", Duration: 30 * time.Minute, Offset: -20 * time.Minute}),
 	g.Entry("order by time asc", helpers.Args{Input: "order_asc", Duration: 25 * time.Minute, Offset: -20 * time.Minute}),
 	g.Entry("order by time desc", helpers.Args{Input: "order_desc", Duration: 25 * time.Minute, Offset: -20 * time.Minute}),
 	g.Entry("order by tag asc", helpers.Args{Input: "order_tag_asc", Duration: 25 * time.Minute, Offset: -20 * time.Minute}),
